// ADC Function
// 2020.01.29 H.Yamada


#include "main.h"
#include "ADC.h"

// Buffer (Common)
uint16_t g_adcBuffer[ADC_CHANNEL_SIZE][ADC_BUFFER_SIZE];
uint8_t  g_adcBufferCount[ADC_CHANNEL_SIZE];
uint8_t  g_adcBufferPos[ADC_CHANNEL_SIZE];

uint16_t g_adcOffset; // offset value of ch0
volatile uint8_t g_adcLock;

void ClearBuffer(uint8_t ch)
{
	g_adcBufferCount[ch] = 0;
	g_adcBufferPos[ch] = 0;
}
void PushBuffer(uint8_t ch, uint16_t value)
{
	if(g_adcLock) return;
	uint8_t i;
	if(ch == 0)
	{
		if(value > g_adcOffset) value -= g_adcOffset;
		else value = 0;
	}
	
	g_adcBuffer[ch][g_adcBufferPos[ch]++] = value;
	if(g_adcBufferPos[ch] >= ADC_BUFFER_SIZE) g_adcBufferPos[ch] = 0;
	
	
	if(g_adcBufferCount[ch] < ADC_BUFFER_SIZE)
	{
		g_adcBufferCount[ch] ++;
	}
}
uint8_t GetBufferCount(uint8_t ch)
{
	return g_adcBufferCount[ch];
}
uint16_t GetBufferMax(uint8_t ch)
{
	uint8_t i;
	uint16_t ret = 0;
	
	for(i=0; i<g_adcBufferCount[ch]; i++)
	{
		if(ret < g_adcBuffer[ch][i]) ret = g_adcBuffer[ch][i];
	}
	return ret;
}
uint16_t GetBufferAvg(uint8_t ch, uint8_t sum)
{
	uint8_t i;
	uint16_t value = 0;
	uint16_t ret;
	
	if(g_adcBufferCount[ch] == 0) return 0;

	LockADCBuffer(1);
	for(i=0; i<g_adcBufferCount[ch]; i++)
	{
		value += g_adcBuffer[ch][i];
	}

	LockADCBuffer(0);
	if(sum) return value;
	
	ret = value / ADC_BUFFER_SIZE;
	return ret;
}

void ADC_Init()
{
	ADMUX  = 0b00000000; // AREF, MUX = ADC0
//	ADCSRA = 0b10001011; // Prescaler 1/8 from 1MHz (128kHz)
	ADCSRA = 0b10001100; // Prescaler 1/16 from 2MHz (128kHz)
}
void EnableADCInterrupt(uint8_t enable)
{
	if(enable) ADCSRA |= 0x08;
	else ADCSRA &= ~0x08;
}
void LockADCBuffer(uint8_t lock)
{
	if(lock) g_adcLock = 1;
	else g_adcLock = 0;
}

// range of current channel
uint8_t g_curChannel;
uint8_t g_curGain; // 0,1,2,3

void SetADCChannel(uint8_t newChannel)
{
	g_curChannel = newChannel;
	uint8_t muxList[ADC_CHANNEL_SIZE] = {0,0,1,1};
	ADMUX = (ADMUX & 0xF0) | (muxList[g_curChannel] & 0x01); // AN10 or AN11
}

uint8_t GetCurrentGainIdx(void)
{
	return g_curGain;
}
float   GetCurrentGain(void)
{
	const float gt[4] = {GAIN_0, GAIN_1, GAIN_2, GAIN_3};
	return gt[GetCurrentGainIdx()];
}


// ADC Interrupt 
ISR(ADC_vect)
{
	static uint8_t ccStableCnt = 255; // wait for stable when range changed
	uint16_t adcValue = 0;
	
	adcValue = ADC; // 10-bit
	PushBuffer(g_curChannel, adcValue);
	if(ccStableCnt != 255){
		ccStableCnt ++;
	}else{
		// Auto Range
		if((g_curChannel == 1) && (GetBufferCount(1) == ADC_BUFFER_SIZE)){
			uint16_t v0 = GetBufferAvg(0, 0);
			uint16_t v1 = GetBufferAvg(1, 0);
			adcValue = (v0 > v1)? v0 - v1: 0;
			if(adcValue < (uint16_t)(1024*GAIN_THRESHOLD_2/VREF_VOLTAGE) )// range under
			{
				if(g_curGain < 3){
					// Range Up 
					g_curGain ++;
					ClearBuffer(0);
					ClearBuffer(1);
					ccStableCnt = -ADC_STABLE_COUNT;
				}
			}else if(adcValue > (uint16_t)(1024*GAIN_THRESHOLD_1/VREF_VOLTAGE)) // range over
			{
				if(g_curGain > 0){
					// Range Down
					g_curGain --;
					ClearBuffer(0);
					ClearBuffer(1);
					ccStableCnt = -ADC_STABLE_COUNT;
				}
			}
		}
	}
	
	// Apply range to GPIO (PC4,5)
	PORTC = (PORTC & 0xCF) | ((g_curGain & 0x03) << 4);
	
	
	if(g_curChannel == ADC_CHANNEL_SIZE-1){
		g_curChannel = 0;
	}else{
		g_curChannel ++;
	}
	SetADCChannel(g_curChannel);
	
	// debug
	CBI(PORTC, PC3);
}

