#ifndef ADC_H_
#define ADC_H_


#define ADC_CHANNEL_SIZE	4
#define ADC_BUFFER_SIZE		64

#define VCC_VOLTAGE 5.0
#define VREF_VOLTAGE 5.0
#define RSRC	1000	// Rs1+Rs2
#define GAIN_FIRST	(82.0/3.30) // first stage

// R3 = 500
#define GAIN_0	(GAIN_FIRST*1.0)
#define GAIN_1	(GAIN_FIRST*9.696)
#define GAIN_2	(GAIN_FIRST*74.33)
#define GAIN_3	(GAIN_FIRST*223)

#define GAIN_THRESHOLD_1 4.1 // volt
#define GAIN_THRESHOLD_2 0.36 // volt
#define OVERRANGE_THRESHOLD 15.0 // ohm (<Vcc/G/2I)
#define ZERO_OFFSET 0.0 // mohm

#define ADC_STABLE_COUNT 40

// Rx = Voltage / Gain / (VCC/(RSRC + Rx))
//    ~= Voltage * RSRC / Gain / Vcc

// offs_value = ZERO_OFFSET * VCC * Gain / RSRC * 1024 / VREF

void ClearBuffer(uint8_t ch);
void PushBuffer(uint8_t ch, uint16_t value);
uint16_t GetBufferAvg(uint8_t ch, uint8_t sum);
uint8_t GetBufferCount(uint8_t ch);

void ADC_Init(void);
void ADC_IRQ(void);

uint8_t GetCurrentRange(void);
uint8_t GetCurrentGainIdx(void);
float   GetCurrentGain(void);
void SetADCChannel(uint8_t newChannel);
void EnableADCInterrupt(uint8_t enable);
void LockADCBuffer(uint8_t lock);

extern uint8_t g_ccRange; // 0,1,2,3

#endif
